package jetbrains.buildServer.inspectionsMonitor;

import java.util.*;

import jetbrains.buildServer.messages.BuildMessage1;
import jetbrains.buildServer.messages.Status;
import jetbrains.buildServer.serverSide.*;
import jetbrains.buildServer.runner.codeInspection.*;

class CodeInspectionBuildListener extends BuildServerAdapter {

    private final SBuildServer myBuildServer;
    private final long myBuildId;

    public CodeInspectionBuildListener(final SBuildServer buildServer, long buildId) {
        myBuildServer = buildServer;
        myBuildId = buildId;
        myBuildServer.addListener(this);
    }

    @Override
    public void beforeBuildFinish(final SRunningBuild build) {
        // If the build is required to be processed by this instance of the listener
        if(build.getBuildId() == myBuildId)  {
            // If the build passed
            if (build.getBuildStatus() == Status.NORMAL) {
                final List<SFinishedBuild> builds = build.getBuildType().getHistory();

                // If this is the first build then don't fail for this reason
                if(!builds.isEmpty()) {
                    // Get the inspection totals for this build
                    InspectionInfo inspectionInfo = new InspectionInfo(myBuildServer, build.getBuildId());
                    int[] stats = inspectionInfo.getStatistics();

                    // Go through the history and find the reference build
                    for (SFinishedBuild finishedBuild : builds) {

                        // If this build passed then use that as the reference
                        if (finishedBuild.getBuildId() != build.getBuildId() && (
                            finishedBuild.getBuildStatus() == Status.NORMAL ||
                            finishedBuild.getTags().contains("INSPECTIONS_BASELINE"))) {

                            // Get the inspection totals for the reference build
                            InspectionInfo referenceInspectionInfo = new InspectionInfo(myBuildServer, finishedBuild.getBuildId());
                            int[] refStats = referenceInspectionInfo.getStatistics();

                            System.out.printf("checking if Build %d failed inspection total %d,%d ref %d total %d %d\n",
                                                build.getBuildId(), stats[0], stats[3], finishedBuild.getBuildId(), refStats[0], refStats[3]);

                            if(stats[0] > refStats[0] ||
                               stats[3] > refStats[3]) {

                                System.out.printf("Build %d failed inspection total %d,%d ref %d total %d %d\n",
                                                    build.getBuildId(), stats[0], stats[3], finishedBuild.getBuildId(), refStats[0], refStats[3]);

                                // Add a new message to the build to inform failure
                                BuildMessage1 bm = new BuildMessage1("DefaultMessage", "Text", Status.FAILURE, new Date(),
                                        String.format("Inspections Monitor : Build configuration failed as inspection total has increased since build %s",
                                                        finishedBuild.getBuildNumber()));
                                build.addBuildMessage(bm);

                                // Set the build to a fail
                                build.setBuildStatus(Status.FAILURE);
                            } else {
                                // Add a new message to the build to inform that a check was performed
                                BuildMessage1 bm = new BuildMessage1("DefaultMessage", "Text", Status.NORMAL, new Date(),
                                        String.format("Inspections Monitor : Build configuration passed as inspection total has not increased since build %s",
                                                        finishedBuild.getBuildNumber()));
                                build.addBuildMessage(bm);
                            }
                            break;

                        }
                    }
                }
            }
            myBuildServer.removeListener(this);
        }
    }
}

public class InspectionsMonitorBuildListener extends BuildServerAdapter {

    private final SBuildServer myBuildServer;

    public InspectionsMonitorBuildListener(SBuildServer sBuildServer) {
        myBuildServer = sBuildServer;
    }

    public void register() {
        myBuildServer.addListener(this);
    }

    @Override
    public void buildStarted(SRunningBuild build) {
        // Create and register a new build listener here in order to be able to modify the build
        // status after the inspections have been totalled up
        new CodeInspectionBuildListener(myBuildServer, build.getBuildId());
    }
}
